/*****************************************************************************
 *                                                                           *
 *     Header for the chain class                                            *
 *     Version 1.0                        Author      :    Henry Koplien     *
 *                                        Create date :    19.11.98          *
 *                                        Last edit   :    27.11.98          *
 *****************************************************************************/
/**
  *  Used to make a chained list of unsigned chars.
  *  The object is build to make an unrestricted list of char's.
  *  The only limitation is the main memory itself. It is mainly used
  *  to read an amount of bytes from a serial port and give them back
  *  in a different length. Not so easy as one might think...
  *  It is all dynamically organized! The first element stays always and
  *  the last ones containing data too.
  *
  *  @author  Henry Koplien
  *  @version 1.0 (first release 19. November 1998, last edit 27. November 1998)
  */
class chain: public sys_message {

private:

  /// Pointer for chaining
  chain          *link;
  /// Pointer for later release to data
  unsigned char  *dcp;
  /// Number of bytes over all
  int            dcp_len;
  /// Pointer to sub-string
  unsigned char  *cp;
  /// Number of bytes left for reading
  int            cp_len;
 
  /**
    *  Shrinks the linked list.
    *  The first object stay and all linked objects containing data's stay too.
    *  All objects between were erased. This method is only private, invoked  
    *  from the read method after copying the data. This avoid deletion of the
    *  very first element.
    *
    *  @return      Pointer to the first object which contains data
    */
  chain          *reduce   (void);

protected:

  /**
    *  Add a string.
    *  This method is used to expand the linked list with the given string.
    *  If nothing is specified only one data is written. The return flags
    *  success or not.
    *
    *  @param  data The pointer to the data
    *  @param  n    How many bytes to write
    *  @return      TRUE or FALSE, depends on the memory allocation
    */
  bool           add       (unsigned char *data, int n=1);
  /**
    *  Read a string.
    *  Reads an amount of data into the by data pointed string. If the gig
    *  fails nothing is internally changed. On success the returned data's were
    *  erased in the linked list. If nothing more is defined, only one char is
    *  returned.
    *
    *  @param  data The pointer for the data to store on success
    *  @param  n    How many bytes requested
    *  @return      TRUE or FALSE, depends on availability of data
    */
  bool           read      (unsigned char *data, int n=1);
  /**
    *  Sizing method.
    *  This method is necessary to get an idea how many bytes are ready to
    *  read.
    *
    *  @return      Number of available data's
    */
  int            available (void);

 public:

  /**
    *  The constructor.
    *  Initialize the own data. Quite easy.
    */
                 chain     (void);
  /**
    *  The destructor.
    *  Release the memory if some was accounted.
    */
                 ~chain    (void);
};
