/*****************************************************************************
 *                                                                           *
 *     Class definition for the linked list                                  *
 *     Version 1.0                        Author      :    Henry Koplien     *
 *                                        Create date :    19.11.98          *
 *                                        Last edit   :    27.11.98          *
 *****************************************************************************/
#include "global.hpp"
#include "mail.hpp"
#include "message.hpp"
#include "chain.hpp"

/***********************************************************************
 *  Name:        Constructor.                                          *
 *  Parameter:   None.                                                 *
 *  Return:      Nothing.                                              *
 *  Function:    Initialize the pointer and length.                    *
 ***********************************************************************/
chain::chain(void) {
  link    = NULL;
  dcp     = NULL;
  cp      = NULL;
  cp_len  = 0;
  dcp_len = 0;
};

/***********************************************************************
 *  Name:        Destructor.                                           *
 *  Parameter:   None.                                                 *
 *  Return:      Nothing.                                              *
 *  Function:    Release the memory if any allocated.                  *
 ***********************************************************************/
chain::~chain(void) {
  if (link)
    delete link;
  if (dcp)
    delete [] dcp;
};

/***********************************************************************
 *  Name:        add.                                                  *
 *  Parameter:   1. Pointer to string.                                 *
 *               2. Number of bytes.                                   *
 *  Return:      TRUE or FALSE.                                        *
 *  Function:    Appends the data to the linked list.                  *
 ***********************************************************************/
bool chain::add(unsigned char *data, int n) {
int i;

  if (data) {
    if (link)
      return link->add(data, n);

    if (dcp) {
      if ((link = new chain) == NULL)
        error(OUT_OF_MEMORY, "chain.add( new object )");
      return link->add(data, n);
    };

    if ((dcp = new unsigned char [n]) == NULL)
      error(OUT_OF_MEMORY, "chain.add( new string )");
    dcp_len = n;
    cp_len  = n;

    cp = dcp;
    for (i = 0; i < n; i++)
      *cp++ = *data++;
    cp = dcp;

    return TRUE;
  };

  return FALSE;
};

/***********************************************************************
 *  Name:        read.                                                 *
 *  Parameter:   1. Pointer to char space.                             *
 *               2. Amount of bytes to read.                           *
 *  Return:      TRUE or FALSE.                                        *
 *  Function:    Reads out data if the requested amount is available.  *
 *               Release the memory after copying as far as possible.  *
 ***********************************************************************/
bool chain::read(unsigned char *data, int n) {
chain *retlink;
bool  result;
int   i;

  if (data == NULL || dcp == NULL || n < 1)
    return FALSE;

  if (available() < n)
    return FALSE;
  
  for (i = 0; cp_len && i < n; i++, cp_len--)
    *data++ = *cp++;

  if (link && i < n) {
    result = link->read(data, n-i);
    if (result) {
      retlink = link->reduce();
      if (retlink != link)
        delete link;
      link = retlink;
    };
    return result;
  };

  return TRUE;
};

/***********************************************************************
 *  Name:        reduce.                                               *
 *  Parameter:   None.                                                 *
 *  Return:      Pointer to first element containing data.             *
 *  Function:    Tries to shrink the linked list. This can't be done   *
 *               everytime and is made with a NULL-Pointer as a termi- *
 *               nator for "read" using the recursive destructor.      *
 ***********************************************************************/
chain *chain::reduce(void) {
chain *retlink;

  if (cp_len)
    return this;

  if (link) {
    retlink = link->reduce();
    if (retlink == link)
      link = NULL;
    return retlink;
  };

  return NULL;
};

/***********************************************************************
 *  Name:        available.                                            *
 *  Parameter:   None.                                                 *
 *  Return:      Size.                                                 *
 *  Function:    Calculate the available number of chars for read out. *
 ***********************************************************************/
int chain::available(void) {
  if (link)
    return link->available()+cp_len;

  return cp_len;
};
