/*****************************************************************************
 *                                                                           *
 *     Class definition for memory management                                *
 *     Version 1.0                        Author      :    Henry Koplien     *
 *                                        Create date :    19.11.98          *
 *                                        Last edit   :    27.11.98          *
 *****************************************************************************/
#include "global.hpp"

static bool epilog     = FALSE;

#ifdef MEMCTRL
static long call_count = 0;
static long size_count = 0;
static long max_count  = 0;
 
/***********************************************************************
 *  Name:        New-Operator-Overloading.                             *
 *  Parameter:   Type size.                                            *
 *  Return:      Pointer to free memory space.                         *
 *  Function:    Redefinition of memory allocation.                    *
 ***********************************************************************/
void *memory::operator new(size_t size) {
  call_count++;
  size_count += size;
  if (size_count > max_count)
    max_count = size_count;
  return ::operator new (size);
};

/***********************************************************************
 *  Name:        Delete-Operator-Overload.                             *
 *  Parameter:   Pointer to allocated memory.                          *
 *  Return:      Nothing.                                              *
 *  Function:    Redefinition of memory delocation.                    *
 ***********************************************************************/
void memory::operator delete (void *pointer_to_bytes, size_t size) {
  call_count--;
  size_count -= size;
  if (epilog && call_count < 5)
    printf("#:%ld, Allocated memory: %ld\n", call_count, size_count);
  ::operator delete (pointer_to_bytes);
};
#endif

/***********************************************************************
 *  Name:        epilog_on.                                            *
 *  Parameter:   None.                                                 *
 *  Return:      Nothing.                                              *
 *  Function:    Enable text output while terminating the programm.    *
 ***********************************************************************/
void memory::epilog_on(void) {
  epilog = TRUE;
};

/***********************************************************************
 *  Name:        status.                                               *
 *  Parameter:   Index how to act.                                     *
 *  Return:      Zero or the maximum of allocated bytes.               *
 *  Function:    Text output of memory allocation.                     *
 ***********************************************************************/
long memory::status(int id) {
#ifdef MEMCTRL
  switch (id) {
    case  1: printf("#:%ld, Allocated memory: %ld\n", call_count, size_count);
             return 0L;
    case  2: printf("Maximal allocated number of bytes: %ld\n", max_count);
             return 0L;
    case  3: return max_count;
    default: printf("Unknown request for <class memory> (id=%d)\n", id);
  };
#else
  printf("Package compiled with no memory watchdog (id=%d)\n", id);
#endif
  return 0L;
};
