/*****************************************************************************
 *                                                                           *
 *     Class definition for messages                                         *
 *     Version 1.1                        Author      :    Henry Koplien     *
 *                                        Create date :    19.11.98          *
 *                                        Last edit   :    29.11.98          *
 *****************************************************************************/
#include "global.hpp"
#include "mail.hpp"
#include "message.hpp"

/***********************************************************************
 *  Name:        constructor.                                          *
 *  Parameter:   None.                                                 *
 *  Return:      Nothing.                                              *
 *  Function:    Set up the entry for the system log file.             *
 *               Every level is reported, except LOG_DEBUG.            *
 ***********************************************************************/
sys_message::sys_message(void) {
  openlog("upsd", LOG_PID, LOG_DAEMON);
  setlogmask(LOG_UPTO(LOG_INFO));
};

/***********************************************************************
 *  Name:        destructor.                                           *
 *  Parameter:   None.                                                 *
 *  Return:      Nothing.                                              *
 *  Function:    Close the system log entry definition.                *
 ***********************************************************************/
sys_message::~sys_message(void){
  closelog();
};

/***********************************************************************
 *  Name:        message.                                              *
 *  Parameter:   String for output.                                    *
 *  Return:      Nothing.                                              *
 *  Function:    Places an information into the log file.              *
 ***********************************************************************/
void sys_message::message(char *text) {
  syslog(LOG_INFO, "UPS-Information: %s\n", text);
};

/***********************************************************************
 *  Name:        warning.                                              *
 *  Parameter:   String for output.                                    *
 *  Return:      Nothing.                                              *
 *  Function:    Places a warning into the log file.                   *
 ***********************************************************************/
void sys_message::warning(char *text) {
  syslog(LOG_WARNING, "UPS-Warning: %s\n", text);
};

/***********************************************************************
 *  Name:        error.                                                *
 *  Parameter:   1. Id.                                                *
 *               2. String for output.                                 *
 *  Return:      Nothing.                                              *
 *  Function:    Places an error into the log file, terminates after   *
 *               logging, except the UPS_REBOOT, which must return for *
 *               the shutdown at higher level. (Other may follow)      *
 ***********************************************************************/
void sys_message::error(error_set id_num, char *text) {
  switch (id_num) {
    case CANT_ADD      : syslog(LOG_ERR, "UPS-Error: Can't add data in <%s>\n", text);
                         send("Can't add data");
                         break;
    case LOCK_FILE     : syslog(LOG_CRIT, "UPS-Connection: Lock file error in <%s>\n", text);
                         send("Lock file");
                         break;
    case OPEN_SIO      : syslog(LOG_CRIT, "UPS-Connection: Open ioctl error in <%s>\n", text);
                         send("Open ioctl");
                         break;
    case READ_SIO      : syslog(LOG_CRIT, "UPS-Connection: Read ioctl error in <%s>\n", text);
                         send("Read ioctl");
                         break;
    case WRITE_SIO     : syslog(LOG_CRIT, "UPS-Connection: Write ioctl error in <%s>\n", text);
                         send("Write ioctl");
                         break;
    case UPS_REBOOT    : syslog(LOG_CRIT, "UPS-Alarm: Rebooting\n");
                         return;
    case RING_ON       : syslog(LOG_CRIT, "UPS-Alarm: Power Failure detected %s\n", text);
                         return;
    case RING_OFF      : syslog(LOG_CRIT, "UPS-Alarm: Power restored %s\n", text);
                         return;
    case IOCTL_ERR     : syslog(LOG_CRIT, "UPS-Connection: Open ioctl error in <Signal-Handler> %s\n", text);
                         return;
    case FORK          : syslog(LOG_ALERT, "UPS-Termination: Fork error in <%s>\n", text);
                         send("Fork error");
                         break;
    case OUT_OF_MEMORY : syslog(LOG_ALERT, "UPS-Termination: Insufficient memory in <%s>\n", text);
                         send("Insufficient memory");
                         break;
    case INTERNAL      : syslog(LOG_ALERT, "UPS-Termination: Found internal error in <%s>\n", text);
                         send("Internal");
                         break;
    default            : syslog(LOG_ALERT, "UPS-Termination: Wrong id_num in <sys_message.error>\n");
                         send("Error number out of range");
  };
  closelog();
  exit(-1);
};
